#pragma once

#ifdef _WIN32
// From winerror.h
#undef NO_ERROR
#endif

#include <vector>
#include <string>

namespace Trueface
{
    /** Error codes returned by methods */
    enum ErrorCode {NO_ERROR, INVALID_LICENSE, FILE_READ_FAIL, UNSUPPORTED_IMAGE_FORMAT, UNSUPPORTED_MODEL, NO_FACE_IN_FRAME, FAILED,
            COLLECTION_CREATION_ERROR, DATABASE_CONNECTION_ERROR, ENROLLMENT_ERROR, MAX_COLLECTION_SIZE_EXCEEDED, NO_RECORD_FOUND};

    /** Image color formats */
    enum ColorCode {bgr, rgb, bgra, rgba, gray, yuv_i420, yuv_nv12};

    /** Object classes */
    enum ObjectLabel {background, aeroplane, bicycle, bird, boat,
                    bottle, bus, car, cat, chair, cow, diningtable,
                    dog, horse, motorbike, person, pottedplant,
                    sheep, sofa, train, tvmonitor};

    /** Face detection modes */
    enum FaceDetectionMode {
        /** The ROBUST mode is optimized for face recognition, smallestFaceHeight less than 80 pixels will be overridden. */
        ROBUST,
        /** The VERSATILE mode can detect small faces (as small as 16x16 pixel) and can be faster than the ROBUST mode. */
        VERSATILE
    };

    /** Face detection filter */
    enum FaceDetectionFilter {
        /** The detected face scores are thresholded to have a low precision and a high recall rate. */
        HIGH_RECALL,
        /** The detected face scores are thresholded to have a high precision and a low recall rate. */
        HIGH_PRECISION,
        /** The detected face scores are thresholded to have a medium precision and recall rate. */
        BALANCED,
        /** The detected face scores are not thresholded. */
        UNFILTERED
    };

    /** Database Management System for storing Faceprints **/
    enum DatabaseManagementSystem {
        /** Use sqlite backend.
         * Write Faceprints to local disk.
         * Ideal for embedded systems or use cases where only one process connects to the database.*/
        SQLITE,
#ifndef _WIN32
        /** Use a PostgreSQL backend.
         * Ideal for distributed systems requiring synchronization.
         * Not available in Windows SDK.**/
        POSTGRESQL,
#endif
        /** Do not write Faceprints to disk, only store in ram.
         * Warning, enrolled Faceprints will not be saved after the program terminates */
        NONE
    };

    /** Facial recognition models */
    enum FacialRecognitionModel {
        /** Lightweight model with faster inference time and reduced length,
         * ideal for embedded systems or CPU only deployments, and for 1 to 1 matching use cases.*/
        LITE,
        /** Full model with improved accuracy. Ideal for GPU deployments, and for 1 to N use cases.*/
        FULL
    };

    /** Object detection models */
    enum ObjectDetectionModel {ACCURATE, FAST};

    /** SDK configuration options */
    struct ConfigurationOptions {
        /** The model to be used for facial recognition (default is LITE model) */
        FacialRecognitionModel frModel = FacialRecognitionModel::LITE;
        /** The model to be used for object detection (default is ACCURATE model) */
        ObjectDetectionModel objModel = ObjectDetectionModel::ACCURATE;
        /**
         * The smallest face height that the face detector can detect. (default is 20 pixels, min value is 16 pixels).
         * The face detector has a detection scale range of about 5 octaves.
         * Ex. 40 pixels yields the detection scale range of ~40 pixels to 1280 (=40x2^5) pixels.
         * If set to -1, will dynamically adjusts the face detection scale range from image-height/32 to image-height
         * to ensure that large faces are detected in high resolution images.
        **/
        int smallestFaceHeight = 20;
        /** The face detection mode (default is VERSATILE) */
        FaceDetectionMode fdMode = FaceDetectionMode::VERSATILE;
        /** The face detection precision-recall mode (default is BALANCED) */
        FaceDetectionFilter fdFilter = FaceDetectionFilter::BALANCED;
        /** Database management system for storing Faceprints (default is SQLITE) **/
        DatabaseManagementSystem dbms = DatabaseManagementSystem::SQLITE;
        /** The directory path containing the model files **/
        std::string modelsPath = "./";
        /** Improves the computation speed for 1 to 1 comparisons and 1 to N searches
         * by compressing the feature vector and enabling additional optimizations. **/
        bool frVectorCompression = false;
        /**
         * Enable GPU support (default is false).
         * Note, GPU support requires a different version of the SDK.
         **/
        bool enableGPU = false;
        /** GPU device index */
        unsigned int deviceIndex = 0;
    };

    template <typename T>
    struct Point { 
        /** Coordinate along the horizontal axis, or pixel column */
        T x; 
        /** Coordinate along the vertical axis, or pixel row */
        T y;
    }; 

    /** Options used when generating the feature vector **/
    struct ModelOptions {
        /** Indicates if the frVectorCompression was enabled when generating the template **/
        bool frVectorCompression = false;
    };

    /** Face feature vector and meta data  **/
    struct Faceprint {
        /** Vector of floats which describe the face **/
        std::vector<float> featureVector;
        /** SDK version used to generate feature vector **/
        std::string sdkVersion;
        /** Name of model used to generate feature vector **/
        std::string modelName;
        /** Additional options used when generating the feature vector **/
        ModelOptions modelOptions;
    };

    struct BoundingBox {
        /** The object class as a label */
        ObjectLabel label;
        /** The detection's confidence probability */
        float probability;
        /** The top left corner of the bounding box */
        Point<float> topLeft;
        /**  The width of the bounding box */
        float width;
        /**  The height of the bounding box */
        float height;
    };

    /** Landmark representing body joint. */
    struct Landmark {
        /** Coordinate of the joint */
        Point<float> point;
        /** Score for the joint */
        float score;
        /** Label for the joint */
        std::string label;
    };

    struct FaceBoxAndLandmarks
    {
        /**  The top left corner of the bounding box */
        Point<float> topLeft;
        /**  The bottom right corner of the bounding box */
        Point<float> bottomRight;
        /**  The facial landmark points: left eye, right eye, nose, left mouth corner, right mouth corner */
        std::vector<Point<float>> landmarks;
        /** Likelihood of this being a true positive; a value lower than 0.85 indicates a high chance of being a false positive */
        float score;
    };

    struct Candidate {
        /** The computed similarity measure */
        float similarityMeasure;
        /** The probability the two face feature vectors are a match */
        float matchProbability;
        /** The identity of the match */
        std::string identity;
        /** The UUID of the match */
        std::string UUID;
    };

    /** Results from spoof detection */
    enum SpoofLabel {
        /** The face image is real */
        REAL,
        /** The image may be an attempted spoof */
        FAKE
    };

    /** Results from mask detection */
    enum MaskLabel {
        /** There is a mask on the face */
        MASK,
        /** There is no mask on the face */
        NO_MASK
    };

    /** Results from eye glasses detection */
    enum GlassesLabel {
        /** There are glasses on the face */
        GLASSES,
        /** There are no glasses on the face */
        NO_GLASSES
    };
}
